package org.dromara.filemanager.controller;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.StrUtil;
import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import org.dromara.filemanager.domain.bo.TblArchivesFileBo;
import org.dromara.filemanager.domain.vo.TblArchivesFileVo;
import org.dromara.filemanager.service.ITblArchivesFileService;
import org.dromara.system.domain.SysOss;
import org.dromara.system.domain.vo.SysOssVo;
import org.dromara.system.service.ISysOssService;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.filemanager.domain.vo.TblArchivesVo;
import org.dromara.filemanager.domain.bo.TblArchivesBo;
import org.dromara.filemanager.service.ITblArchivesService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 电子档案
 *
 * @author Lion Li
 * @date 2024-05-20
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/filemanager/archives")
public class TblArchivesController extends BaseController {

    private final ITblArchivesService tblArchivesService;

    private final ITblArchivesFileService   tblArchivesFileService;


    private final ISysOssService sysOssService;

    /**
     * 查询电子档案列表
     */
    @GetMapping("/list")
    public TableDataInfo<TblArchivesVo> list(TblArchivesBo bo, PageQuery pageQuery) {
        return tblArchivesService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出电子档案列表
     */
    @Log(title = "电子档案", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(TblArchivesBo bo, HttpServletResponse response) {
        List<TblArchivesVo> list = tblArchivesService.queryList(bo);
        ExcelUtil.exportExcel(list, "电子档案", TblArchivesVo.class, response);
    }

    /**
     * 获取电子档案详细信息
     *
     * @param id 主键
     */
    @GetMapping("/{id}")
    public R<TblArchivesVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(tblArchivesService.queryById(id));
    }

    /**
     * 新增电子档案
     */
    @Log(title = "电子档案", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody TblArchivesBo bo) {
        TblArchivesBo b = new TblArchivesBo();
        List<TblArchivesVo> list =  tblArchivesService.queryList(new TblArchivesBo());
        if(list.size()>0){
            if(list.stream().filter(t->t.getQiuquan()!=null && t.getEcode()!=null &&( t.getQiuquan().equals(bo.getQiuquan())||t.getEcode().equals(bo.getEcode()))).toList().size()>0){
                return R.fail("存在相同丘权或电子编号！");
            }
        }

        return toAjax(tblArchivesService.insertByBo(bo));
    }

    /**
     * 修改电子档案
     */
    @Log(title = "电子档案", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody TblArchivesBo bo) {
        return toAjax(tblArchivesService.updateByBo(bo));
    }

    /**
     * 删除电子档案
     *
     * @param ids 主键串
     */
    @Log(title = "电子档案", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(tblArchivesService.deleteWithValidByIds(List.of(ids), true));
    }

    /**
     * 查询电子档案文件列表
     *
     * @param id 主键
     */
    @GetMapping("/{id}/files")
    public R<List<SysOssVo>> files(@PathVariable Long id) {
        TblArchivesFileBo bo = new TblArchivesFileBo();
        bo.setArchiveId(id);
        List<TblArchivesFileVo> afiles =  tblArchivesFileService.queryList(bo);
        List<SysOssVo> files =  sysOssService.listByIds(afiles.stream().map(t->t.getOssFileId()).toList());
        return R.ok(files);
    }

    @PutMapping("/save/{id}/{fileids}")
    public R<Void> savefiles(@PathVariable Long id,@PathVariable String fileids) {
        if(!StrUtil.isEmptyIfStr(fileids)){
          List<TblArchivesFileBo> fileBoList =  Arrays.stream(fileids.split(",")).map(t->{
                TblArchivesFileBo bo = new TblArchivesFileBo();
                bo.setArchiveId(id);
                bo.setOssFileId(Long.parseLong(t));
                return bo;
            }).toList();
          tblArchivesFileService.insertBatch(fileBoList);
        }else{
            return R.fail("缺少文件！");
        }
        return R.ok();
    }
}
